@@ -1,5 +1,5 @@
 /* cat -- concatenate files and print on the standard output.
-   Copyright (C) 88, 90, 91, 1995-2004 Free Software Foundation, Inc.
+   Copyright (C) 88, 90, 91, 1995-2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,26 +13,33 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /* Differences from the Unix cat:
    * Always unbuffered, -u is ignored.
    * Usually much faster than other versions of cat, the difference
    is especially apparent when using the -v option.
 
-   By tege@sics.se, Torbjorn Granlund, advised by rms, Richard Stallman. */
+   By tege@sics.se, Torbjorn Granlund, advised by rms, Richard Stallman.  */
 
 #include <config.h>
 
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
-#ifndef _POSIX_SOURCE
+
+#if HAVE_STROPTS_H
+# include <stropts.h>
+#endif
+#if HAVE_SYS_IOCTL_H
 # include <sys/ioctl.h>
 #endif
+
 #include "system.h"
 #include "error.h"
 #include "full-write.h"
+#include "getpagesize.h"
+#include "quote.h"
 #include "safe-read.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
@@ -77,9 +84,6 @@ static char *line_num_end = line_buf + LINE_COUNTER_BUF_LEN - 3;
 /* Preserves the `cat' function's local `newlines' between invocations.  */
 static int newlines2 = 0;
 
-/* Nonzero if a non-fatal error has occurred.  */
-static int exit_status = 0;
-
 void
 usage (int status)
 {
@@ -114,12 +118,13 @@ Concatenate FILE(s), or standard input, to standard output.\n\
 \n\
 With no FILE, or when FILE is -, read standard input.\n\
 "), stdout);
-#if O_BINARY
-      fputs (_("\
+      printf (_("\
 \n\
-  -B, --binary             use binary writes to the console device.\n\n\
-"), stdout);
-#endif
+Examples:\n\
+  %s f - g  Output f's contents, then standard input, then g's contents.\n\
+  %s        Copy standard input to standard output.\n\
+"),
+	      program_name, program_name);
       printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
@@ -146,16 +151,17 @@ next_line_num (void)
     line_num_print--;
 }
 
-/* Plain cat.  Copies the file behind `input_desc' to STDOUT_FILENO.  */
+/* Plain cat.  Copies the file behind `input_desc' to STDOUT_FILENO.
+   Return true if successful.  */
 
-static void
+static bool
 simple_cat (
      /* Pointer to the buffer, used by reads and writes.  */
      char *buf,
 
      /* Number of characters preferably read or written by each read and write
         call.  */
-     int bufsize)
+     size_t bufsize)
 {
   /* Actual number of characters read, and therefore written.  */
   size_t n_read;
@@ -170,14 +176,13 @@ simple_cat (
       if (n_read == SAFE_READ_ERROR)
 	{
 	  error (0, errno, "%s", infile);
-	  exit_status = 1;
-	  return;
+	  return false;
 	}
 
       /* End of this file?  */
 
       if (n_read == 0)
-	break;
+	return true;
 
       /* Write this block out.  */
 
@@ -191,12 +196,13 @@ simple_cat (
 }
 
 /* Cat the file behind INPUT_DESC to the file behind OUTPUT_DESC.
+   Return true if successful.
    Called if any option more than -u was specified.
 
    A newline character is always put at the end of the buffer, to make
    an explicit test for buffer end unnecessary.  */
 
-static void
+static bool
 cat (
      /* Pointer to the beginning of the input buffer.  */
      char *inbuf,
@@ -211,12 +217,12 @@ cat (
      size_t outsize,
 
      /* Variables that have values according to the specified options.  */
-     int quote,
-     int output_tabs,
-     int numbers,
-     int numbers_at_empty_lines,
-     int mark_line_ends,
-     int squeeze_empty_lines)
+     bool show_nonprinting,
+     bool show_tabs,
+     bool number,
+     bool number_nonblank,
+     bool show_ends,
+     bool squeeze_blank)
 {
   /* Last character read from the input buffer.  */
   unsigned char ch;
@@ -243,8 +249,8 @@ cat (
 
 #ifdef FIONREAD
   /* If nonzero, use the FIONREAD ioctl, as an optimization.
-     (On Ultrix, it is not supported on NFS filesystems.)  */
-  int use_fionread = 1;
+     (On Ultrix, it is not supported on NFS file systems.)  */
+  bool use_fionread = true;
 #endif
 
   /* The inbuf pointers are initialized so that BPIN > EOB, and thereby input
@@ -304,13 +310,12 @@ cat (
 		  if (errno == EOPNOTSUPP || errno == ENOTTY
 		      || errno == EINVAL || errno == ENODEV
 		      || errno == ENOSYS)
-		    use_fionread = 0;
+		    use_fionread = false;
 		  else
 		    {
-		      error (0, errno, _("cannot do ioctl on `%s'"), infile);
-		      exit_status = 1;
+		      error (0, errno, _("cannot do ioctl on %s"), quote (infile));
 		      newlines2 = newlines;
-		      return;
+		      return false;
 		    }
 		}
 	      if (n_to_read == 0)
@@ -329,14 +334,13 @@ cat (
 	      if (n_read == SAFE_READ_ERROR)
 		{
 		  error (0, errno, "%s", infile);
-		  exit_status = 1;
 		  newlines2 = newlines;
-		  return;
+		  return false;
 		}
 	      if (n_read == 0)
 		{
 		  newlines2 = newlines;
-		  return;
+		  return true;
 		}
 
 	      /* Update the pointers and insert a sentinel at the buffer
@@ -365,7 +369,7 @@ cat (
 		      /* Are multiple adjacent empty lines to be substituted
 			 by single ditto (-s), and this was the second empty
 			 line?  */
-		      if (squeeze_empty_lines)
+		      if (squeeze_blank)
 			{
 			  ch = *bpin++;
 			  continue;
@@ -374,7 +378,7 @@ cat (
 
 		  /* Are line numbers to be written at empty lines (-n)?  */
 
-		  if (numbers && numbers_at_empty_lines)
+		  if (number & !number_nonblank)
 		    {
 		      next_line_num ();
 		      bpout = stpcpy (bpout, line_num_print);
@@ -383,7 +387,7 @@ cat (
 
 	      /* Output a currency symbol if requested (-e).  */
 
-	      if (mark_line_ends)
+	      if (show_ends)
 		*bpout++ = '$';
 
 	      /* Output the newline.  */
@@ -396,7 +400,7 @@ cat (
 
       /* Are we at the beginning of a line, and line numbers are requested?  */
 
-      if (newlines >= 0 && numbers)
+      if (newlines >= 0 && number)
 	{
 	  next_line_num ();
 	  bpout = stpcpy (bpout, line_num_print);
@@ -410,7 +414,7 @@ cat (
 
       /* If quoting, i.e. at least one of -v, -e, or -t specified,
 	 scan for chars that need conversion.  */
-      if (quote)
+      if (show_nonprinting)
 	{
 	  for (;;)
 	    {
@@ -444,7 +448,7 @@ cat (
 			}
 		    }
 		}
-	      else if (ch == '\t' && output_tabs)
+	      else if (ch == '\t' && !show_tabs)
 		*bpout++ = '\t';
 	      else if (ch == '\n')
 		{
@@ -465,7 +469,7 @@ cat (
 	  /* Not quoting, neither of -v, -e, or -t specified.  */
 	  for (;;)
 	    {
-	      if (ch == '\t' && !output_tabs)
+	      if (ch == '\t' && show_tabs)
 		{
 		  *bpout++ = '^';
 		  *bpout++ = ch + 64;
@@ -484,17 +488,6 @@ cat (
     }
 }
 
-/* This is gross, but necessary, because of the way close_stdout
-   works and because this program closes STDOUT_FILENO directly.  */
-static void (*closeout_func) (void) = close_stdout;
-
-static void
-close_stdout_wrapper (void)
-{
-  if (closeout_func)
-    (*closeout_func) ();
-}
-
 int
 main (int argc, char **argv)
 {
@@ -504,12 +497,15 @@ main (int argc, char **argv)
   /* Optimal size of i/o operations of input.  */
   size_t insize;
 
+  size_t page_size = getpagesize ();
+
   /* Pointer to the input buffer.  */
   char *inbuf;
 
   /* Pointer to the output buffer.  */
   char *outbuf;
 
+  bool ok = true;
   int c;
 
   /* Index in argv to processed argument.  */
@@ -521,30 +517,23 @@ main (int argc, char **argv)
   /* I-node number of the output.  */
   ino_t out_ino;
 
-  /* Nonzero if the output file should not be the same as any input file. */
-  int check_redirection = 1;
+  /* True if the output file should not be the same as any input file.  */
+  bool check_redirection = true;
 
-  /* Nonzero if we have ever read standard input. */
-  int have_read_stdin = 0;
+  /* Nonzero if we have ever read standard input.  */
+  bool have_read_stdin = false;
 
   struct stat stat_buf;
 
   /* Variables that are set according to the specified options.  */
-  int numbers = 0;
-  int numbers_at_empty_lines = 1;
-  int squeeze_empty_lines = 0;
-  int mark_line_ends = 0;
-  int quote = 0;
-  int output_tabs = 1;
-#if O_BINARY
-  int binary_files  = 0;
-  int binary_output = 0;
-#endif
+  bool number = false;
+  bool number_nonblank = false;
+  bool squeeze_blank = false;
+  bool show_ends = false;
+  bool show_nonprinting = false;
+  bool show_tabs = false;
   int file_open_mode = O_RDONLY;
 
-/* If nonzero, call cat, otherwise call simple_cat to do the actual work. */
-  int options = 0;
-
   static struct option const long_options[] =
   {
     {"number-nonblank", no_argument, NULL, 'b'},
@@ -554,9 +543,6 @@ main (int argc, char **argv)
     {"show-ends", no_argument, NULL, 'E'},
     {"show-tabs", no_argument, NULL, 'T'},
     {"show-all", no_argument, NULL, 'A'},
-#if O_BINARY
-    {"binary", no_argument, NULL, 'B'},
-#endif
     {GETOPT_HELP_OPTION_DECL},
     {GETOPT_VERSION_OPTION_DECL},
     {NULL, 0, NULL, 0}
@@ -570,49 +556,36 @@ main (int argc, char **argv)
 
   /* Arrange to close stdout if we exit via the
      case_GETOPT_HELP_CHAR or case_GETOPT_VERSION_CHAR code.  */
-  atexit (close_stdout_wrapper);
+  atexit (close_stdout);
 
   /* Parse command line options.  */
 
-  while ((c = getopt_long (argc, argv,
-#if O_BINARY
-			   "benstuvABET"
-#else
-			   "benstuvAET"
-#endif
-			   , long_options, NULL)) != -1)
+  while ((c = getopt_long (argc, argv, "benstuvAET", long_options, NULL))
+	 != -1)
     {
       switch (c)
 	{
-	case 0:
-	  break;
-
 	case 'b':
-	  ++options;
-	  numbers = 1;
-	  numbers_at_empty_lines = 0;
+	  number = true;
+	  number_nonblank = true;
 	  break;
 
 	case 'e':
-	  ++options;
-	  mark_line_ends = 1;
-	  quote = 1;
+	  show_ends = true;
+	  show_nonprinting = true;
 	  break;
 
 	case 'n':
-	  ++options;
-	  numbers = 1;
+	  number = true;
 	  break;
 
 	case 's':
-	  ++options;
-	  squeeze_empty_lines = 1;
+	  squeeze_blank = true;
 	  break;
 
 	case 't':
-	  ++options;
-	  output_tabs = 0;
-	  quote = 1;
+	  show_tabs = true;
+	  show_nonprinting = true;
 	  break;
 
 	case 'u':
@@ -620,32 +593,21 @@ main (int argc, char **argv)
 	  break;
 
 	case 'v':
-	  ++options;
-	  quote = 1;
+	  show_nonprinting = true;
 	  break;
 
 	case 'A':
-	  ++options;
-	  quote = 1;
-	  mark_line_ends = 1;
-	  output_tabs = 0;
+	  show_nonprinting = true;
+	  show_ends = true;
+	  show_tabs = true;
 	  break;
 
-#if O_BINARY
-	case 'B':
-	  ++options;
-	  binary_files = 1;
-	  break;
-#endif
-
 	case 'E':
-	  ++options;
-	  mark_line_ends = 1;
+	  show_ends = true;
 	  break;
 
 	case 'T':
-	  ++options;
-	  output_tabs = 0;
+	  show_tabs = true;
 	  break;
 
 	case_GETOPT_HELP_CHAR;
@@ -657,9 +619,6 @@ main (int argc, char **argv)
 	}
     }
 
-  /* Don't close stdout on exit from here on.  */
-  closeout_func = NULL;
-
   /* Get device, i-node number, and optimal blocksize of output.  */
 
   if (fstat (STDOUT_FILENO, &stat_buf) < 0)
@@ -670,7 +629,7 @@ main (int argc, char **argv)
      fstat on pipes returns S_IFSOCK on some systems, S_IFIFO
      on others, so the checking should not be done for those types,
      and to allow things like cat < /dev/tty > /dev/tty, checking
-     is not done for device files either. */
+     is not done for device files either.  */
 
   if (S_ISREG (stat_buf.st_mode))
     {
@@ -679,48 +638,19 @@ main (int argc, char **argv)
     }
   else
     {
-      check_redirection = 0;
+      check_redirection = false;
 #ifdef lint  /* Suppress `used before initialized' warning.  */
       out_dev = 0;
       out_ino = 0;
 #endif
     }
 
-#if O_BINARY
-  /* We always read and write in BINARY mode, since this is the
-     best way to copy the files verbatim.  Exceptions are when
-     they request line numbering, squeezing of empty lines or
-     marking lines' ends: then we use text I/O, because otherwise
-     -b, -s and -E would surprise users on DOS/Windows where a line
-     with only CR-LF is an empty line.  (Besides, if they ask for
-     one of these options, they don't care much about the original
-     file contents anyway).  */
-  if ((!isatty (STDOUT_FILENO)
-       && !(numbers || squeeze_empty_lines || mark_line_ends))
-      || binary_files)
+  if (! (number | show_ends | squeeze_blank))
     {
-      /* Switch stdout to BINARY mode.  */
-      binary_output = 1;
-      SET_BINARY (STDOUT_FILENO);
-      /* When stdout is in binary mode, make sure all input files are
-	 also read in binary mode.  */
       file_open_mode |= O_BINARY;
+      if (O_BINARY && ! isatty (STDOUT_FILENO))
+	freopen (NULL, "wb", stdout);
     }
-  else if (quote)
-    {
-      /* If they want to see the non-printables, let's show them
-	 those CR characters as well, so make the input binary.
-	 But keep console output in text mode, so that LF causes
-	 both CR and LF on output, and the output is readable.  */
-      file_open_mode |= O_BINARY;
-      SET_BINARY (0);
-
-      /* Setting stdin to binary switches the console device to
-	 raw I/O, which also affects stdout to console.  Undo that.  */
-      if (isatty (STDOUT_FILENO))
-	setmode (STDOUT_FILENO, O_TEXT);
-    }
-#endif
 
   /* Check if any of the input files are the same as the output file.  */
 
@@ -734,37 +664,12 @@ main (int argc, char **argv)
       if (argind < argc)
 	infile = argv[argind];
 
-      if (infile[0] == '-' && infile[1] == 0)
+      if (STREQ (infile, "-"))
 	{
-	  have_read_stdin = 1;
-	  input_desc = 0;
-
-#if O_BINARY
-	  /* Switch stdin to BINARY mode if needed.  */
-	  if (binary_output)
-	    {
-	      int tty_in = isatty (input_desc);
-
-	      /* If stdin is a terminal device, and it is the ONLY
-		 input file (i.e. we didn't write anything to the
-		 output yet), switch the output back to TEXT mode.
-		 This is so "cat > xyzzy" creates a DOS-style text
-		 file, like people expect.  */
-	      if (tty_in && optind <= argc)
-		setmode (STDOUT_FILENO, O_TEXT);
-	      else
-		{
-		  SET_BINARY (input_desc);
-# ifdef __DJGPP__
-		  /* This is DJGPP-specific.  By default, switching console
-		     to binary mode disables SIGINT.  But we want terminal
-		     reads to be interruptible.  */
-		  if (tty_in)
-		    __djgpp_set_ctrl_c (1);
-# endif
-		}
-	    }
-#endif
+	  have_read_stdin = true;
+	  input_desc = STDIN_FILENO;
+	  if ((file_open_mode & O_BINARY) && ! isatty (STDIN_FILENO))
+	    freopen (NULL, "rb", stdin);
 	}
       else
 	{
@@ -772,7 +677,7 @@ main (int argc, char **argv)
 	  if (input_desc < 0)
 	    {
 	      error (0, errno, "%s", infile);
-	      exit_status = 1;
+	      ok = false;
 	      continue;
 	    }
 	}
@@ -780,7 +685,7 @@ main (int argc, char **argv)
       if (fstat (input_desc, &stat_buf) < 0)
 	{
 	  error (0, errno, "%s", infile);
-	  exit_status = 1;
+	  ok = false;
 	  goto contin;
 	}
       insize = ST_BLKSIZE (stat_buf);
@@ -795,26 +700,27 @@ main (int argc, char **argv)
 	  && (input_desc != STDIN_FILENO))
 	{
 	  error (0, 0, _("%s: input file is output file"), infile);
-	  exit_status = 1;
+	  ok = false;
 	  goto contin;
 	}
 
-      /* Select which version of `cat' to use. If any options (more than -u,
-	 --version, or --help) were specified, use `cat', otherwise use
-	 `simple_cat'.  */
+      /* Select which version of `cat' to use.  If any format-oriented
+	 options were given use `cat'; otherwise use `simple_cat'.  */
 
-      if (options == 0)
+      if (! (number | show_ends | show_nonprinting
+	     | show_tabs | squeeze_blank))
 	{
 	  insize = max (insize, outsize);
-	  inbuf = xmalloc (insize);
+	  inbuf = xmalloc (insize + page_size - 1);
 
-	  simple_cat (inbuf, insize);
+	  ok &= simple_cat (ptr_align (inbuf, page_size), insize);
 	}
       else
 	{
-	  inbuf = xmalloc (insize + 1);
+	  inbuf = xmalloc (insize + 1 + page_size - 1);
 
-	  /* Why are (OUTSIZE - 1 + INSIZE * 4 + LINE_COUNTER_BUF_LEN)
+	  /* Why are
+	     (OUTSIZE - 1 + INSIZE * 4 + LINE_COUNTER_BUF_LEN + PAGE_SIZE - 1)
 	     bytes allocated for the output buffer?
 
 	     A test whether output needs to be written is done when the input
@@ -829,13 +735,19 @@ main (int argc, char **argv)
 	     options) as the first thing in the output buffer. (Done after the
 	     new input is read, but before processing of the input begins.)
 	     A line number requires seldom more than LINE_COUNTER_BUF_LEN
-	     positions.  */
+	     positions.
 
-	  outbuf = xmalloc (outsize - 1 + insize * 4 + LINE_COUNTER_BUF_LEN);
+	     Align the output buffer to a page size boundary, for efficency on
+	     some paging implementations, so add PAGE_SIZE - 1 bytes to the
+	     request to make room for the alignment.  */
 
-	  cat (inbuf, insize, outbuf, outsize, quote,
-	       output_tabs, numbers, numbers_at_empty_lines, mark_line_ends,
-	       squeeze_empty_lines);
+	  outbuf = xmalloc (outsize - 1 + insize * 4 + LINE_COUNTER_BUF_LEN
+			    + page_size - 1);
+
+	  ok &= cat (ptr_align (inbuf, page_size), insize,
+		     ptr_align (outbuf, page_size), outsize, show_nonprinting,
+		     show_tabs, number, number_nonblank, show_ends,
+		     squeeze_blank);
 
 	  free (outbuf);
 	}
@@ -846,7 +758,7 @@ main (int argc, char **argv)
       if (!STREQ (infile, "-") && close (input_desc) < 0)
 	{
 	  error (0, errno, "%s", infile);
-	  exit_status = 1;
+	  ok = false;
 	}
     }
   while (++argind < argc);
@@ -854,8 +766,5 @@ main (int argc, char **argv)
   if (have_read_stdin && close (STDIN_FILENO) < 0)
     error (EXIT_FAILURE, errno, _("closing standard input"));
 
-  if (close (STDOUT_FILENO) < 0)
-    error (EXIT_FAILURE, errno, _("closing standard output"));
-
-  exit (exit_status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
+  exit (ok ? EXIT_SUCCESS : EXIT_FAILURE);
 }
