@@ -1,5 +1,5 @@
 /* tee - read from standard input and write to standard output and files.
-   Copyright (C) 85,1990-2004 Free Software Foundation, Inc.
+   Copyright (C) 85,1990-2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,31 +13,31 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /* Mike Parker, Richard M. Stallman, and David MacKenzie */
 
 #include <config.h>
-#include <stdio.h>
 #include <sys/types.h>
 #include <signal.h>
 #include <getopt.h>
 
 #include "system.h"
 #include "error.h"
+#include "stdio--.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "tee"
 
 #define AUTHORS "Mike Parker", "Richard M. Stallman", "David MacKenzie"
 
-static int tee (int nfiles, const char **files);
+static bool tee (int nfiles, const char **files);
 
-/* If nonzero, append to output files rather than truncating them. */
-static int append;
+/* If true, append to output files rather than truncating them. */
+static bool append;
 
-/* If nonzero, ignore interrupts. */
-static int ignore_interrupts;
+/* If true, ignore interrupts. */
+static bool ignore_interrupts;
 
 /* The name that this program was run with. */
 char *program_name;
@@ -68,6 +68,10 @@ Copy standard input to each FILE, and also to standard output.\n\
 "), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
+\n\
+If a FILE is -, copy again to standard output.\n\
+"), stdout);
       printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
@@ -76,7 +80,7 @@ Copy standard input to each FILE, and also to standard output.\n\
 int
 main (int argc, char **argv)
 {
-  int errs;
+  bool ok;
   int optc;
 
   initialize_main (&argc, &argv);
@@ -87,22 +91,19 @@ main (int argc, char **argv)
 
   atexit (close_stdout);
 
-  append = 0;
-  ignore_interrupts = 0;
+  append = false;
+  ignore_interrupts = false;
 
   while ((optc = getopt_long (argc, argv, "ai", long_options, NULL)) != -1)
     {
       switch (optc)
 	{
-	case 0:
-	  break;
-
 	case 'a':
-	  append = 1;
+	  append = true;
 	  break;
 
 	case 'i':
-	  ignore_interrupts = 1;
+	  ignore_interrupts = true;
 	  break;
 
 	case_GETOPT_HELP_CHAR;
@@ -115,56 +116,34 @@ main (int argc, char **argv)
     }
 
   if (ignore_interrupts)
-    {
-#ifdef _POSIX_SOURCE
-      struct sigaction sigact;
-
-      sigact.sa_handler = SIG_IGN;
-      sigemptyset (&sigact.sa_mask);
-      sigact.sa_flags = 0;
-      sigaction (SIGINT, &sigact, NULL);
-#else				/* !_POSIX_SOURCE */
-      signal (SIGINT, SIG_IGN);
-#endif				/* _POSIX_SOURCE */
-    }
-
-  /* Don't let us be killed if one of the output files is a pipe that
-     doesn't consume all its input.  */
-#ifdef _POSIX_SOURCE
-  {
-    struct sigaction sigact;
-
-    sigact.sa_handler = SIG_IGN;
-    sigemptyset (&sigact.sa_mask);
-    sigact.sa_flags = 0;
-    sigaction (SIGPIPE, &sigact, NULL);
-  }
-#else
-  signal (SIGPIPE, SIG_IGN);
-#endif
+    signal (SIGINT, SIG_IGN);
 
   /* Do *not* warn if tee is given no file arguments.
      POSIX requires that it work when given no arguments.  */
 
-  errs = tee (argc - optind, (const char **) &argv[optind]);
+  ok = tee (argc - optind, (const char **) &argv[optind]);
   if (close (STDIN_FILENO) != 0)
     error (EXIT_FAILURE, errno, _("standard input"));
 
-  exit (errs);
+  exit (ok ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Copy the standard input into each of the NFILES files in FILES
    and into the standard output.
-   Return 0 if successful, 1 if any errors occur. */
+   Return true if successful.  */
 
-static int
+static bool
 tee (int nfiles, const char **files)
 {
   FILE **descriptors;
   char buffer[BUFSIZ];
-  int bytes_read, i;
-  int ret = 0;
-  const char *mode_string = (append ? "a" : "w");
+  ssize_t bytes_read;
+  int i;
+  bool ok = true;
+  char const *mode_string =
+    (O_BINARY
+     ? (append ? "ab" : "wb")
+     : (append ? "a" : "w"));
 
   descriptors = xnmalloc (nfiles + 1, sizeof *descriptors);
 
@@ -173,7 +152,10 @@ tee (int nfiles, const char **files)
   for (i = nfiles; i >= 1; i--)
     files[i] = files[i - 1];
 
-  SET_BINARY2 (0, 1);
+  if (O_BINARY && ! isatty (STDIN_FILENO))
+    freopen (NULL, "rb", stdin);
+  if (O_BINARY && ! isatty (STDOUT_FILENO))
+    freopen (NULL, "wb", stdout);
 
   /* In the array of NFILES + 1 descriptors, make
      the first one correspond to standard output.   */
@@ -183,17 +165,16 @@ tee (int nfiles, const char **files)
 
   for (i = 1; i <= nfiles; i++)
     {
-      descriptors[i] = fopen (files[i], mode_string);
+      descriptors[i] = (STREQ (files[i], "-")
+			? stdout
+			: fopen (files[i], mode_string));
       if (descriptors[i] == NULL)
 	{
 	  error (0, errno, "%s", files[i]);
-	  ret = 1;
+	  ok = false;
 	}
       else
-	{
-	  SETVBUF (descriptors[i], NULL, _IONBF, 0);
-	  SET_BINARY (fileno (descriptors[i]));
-	}
+	SETVBUF (descriptors[i], NULL, _IONBF, 0);
     }
 
   while (1)
@@ -214,25 +195,26 @@ tee (int nfiles, const char **files)
 	  {
 	    error (0, errno, "%s", files[i]);
 	    descriptors[i] = NULL;
-	    ret = 1;
+	    ok = false;
 	  }
     }
 
   if (bytes_read == -1)
     {
       error (0, errno, _("read error"));
-      ret = 1;
+      ok = false;
     }
 
   /* Close the files, but not standard output.  */
   for (i = 1; i <= nfiles; i++)
-    if (descriptors[i] && fclose (descriptors[i]) != 0)
+    if (!STREQ (files[i], "-")
+	&& descriptors[i] && fclose (descriptors[i]) != 0)
       {
 	error (0, errno, "%s", files[i]);
-	ret = 1;
+	ok = false;
       }
 
   free (descriptors);
 
-  return ret;
+  return ok;
 }
