@@ -1,5 +1,5 @@
 /* uniq -- remove duplicate lines from a sorted file
-   Copyright (C) 86, 91, 1995-2004 Free Software Foundation, Inc.
+   Copyright (C) 86, 91, 1995-2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /* Written by Richard Stallman and David MacKenzie. */
 
@@ -29,6 +29,7 @@
 #include "error.h"
 #include "hard-locale.h"
 #include "posixver.h"
+#include "quote.h"
 #include "xmemcoll.h"
 #include "xstrtol.h"
 #include "memcasecmp.h"
@@ -51,8 +52,8 @@
 /* The name this program was run with. */
 char *program_name;
 
-/* Nonzero if the LC_COLLATE locale is hard.  */
-static int hard_LC_COLLATE;
+/* True if the LC_COLLATE locale is hard.  */
+static bool hard_LC_COLLATE;
 
 /* Number of fields to skip on each line when doing comparisons. */
 static size_t skip_fields;
@@ -80,8 +81,8 @@ static bool output_unique;
 static bool output_first_repeated;
 static bool output_later_repeated;
 
-/* If nonzero, ignore case when comparing.  */
-static int ignore_case;
+/* If true, ignore case when comparing.  */
+static bool ignore_case;
 
 enum delimit_method
 {
@@ -97,7 +98,7 @@ enum delimit_method
 
 static char const *const delimit_method_string[] =
 {
-  "none", "prepend", "separate", 0
+  "none", "prepend", "separate", NULL
 };
 
 static enum delimit_method const delimit_method_map[] =
@@ -190,10 +191,10 @@ size_opt (char const *opt, char const *msgid)
 static char *
 find_field (const struct linebuffer *line)
 {
-  register size_t count;
-  register char *lp = line->buffer;
-  register size_t size = line->length - 1;
-  register size_t i = 0;
+  size_t count;
+  char *lp = line->buffer;
+  size_t size = line->length - 1;
+  size_t i = 0;
 
   for (count = 0; count < skip_fields && i < size; count++)
     {
@@ -209,12 +210,12 @@ find_field (const struct linebuffer *line)
   return lp + i;
 }
 
-/* Return zero if two strings OLD and NEW match, nonzero if not.
+/* Return false if two strings OLD and NEW match, true if not.
    OLD and NEW point not to the beginnings of the lines
    but rather to the beginnings of the fields to compare.
    OLDLEN and NEWLEN are their lengths. */
 
-static int
+static bool
 different (char *old, char *new, size_t oldlen, size_t newlen)
 {
   if (check_chars < oldlen)
@@ -227,21 +228,21 @@ different (char *old, char *new, size_t oldlen, size_t newlen)
       /* FIXME: This should invoke strcoll somehow.  */
       return oldlen != newlen || memcasecmp (old, new, oldlen);
     }
-  else if (HAVE_SETLOCALE && hard_LC_COLLATE)
-    return xmemcoll (old, oldlen, new, newlen);
+  else if (hard_LC_COLLATE)
+    return xmemcoll (old, oldlen, new, newlen) != 0;
   else
     return oldlen != newlen || memcmp (old, new, oldlen);
 }
 
-/* Output the line in linebuffer LINE to stream STREAM
+/* Output the line in linebuffer LINE to standard output
    provided that the switches say it should be output.
    MATCH is true if the line matches the previous line.
    If requested, print the number of times it occurred, as well;
    LINECOUNT + 1 is the number of times that the line occurred. */
 
 static void
-writeline (struct linebuffer const *line, FILE *stream,
-	   bool match, int linecount)
+writeline (struct linebuffer const *line,
+	   bool match, uintmax_t linecount)
 {
   if (! (linecount == 0 ? output_unique
 	 : !match ? output_first_repeated
@@ -249,9 +250,9 @@ writeline (struct linebuffer const *line, FILE *stream,
     return;
 
   if (countmode == count_occurrences)
-    fprintf (stream, "%7d ", linecount + 1);
+    printf ("%7" PRIuMAX " ", linecount + 1);
 
-  fwrite (line->buffer, sizeof (char), line->length, stream);
+  fwrite (line->buffer, sizeof (char), line->length, stdout);
 }
 
 /* Process input file INFILE with output to OUTFILE.
@@ -260,23 +261,12 @@ writeline (struct linebuffer const *line, FILE *stream,
 static void
 check_file (const char *infile, const char *outfile)
 {
-  FILE *istream;
-  FILE *ostream;
   struct linebuffer lb1, lb2;
   struct linebuffer *thisline, *prevline;
 
-  if (STREQ (infile, "-"))
-    istream = stdin;
-  else
-    istream = fopen (infile, "r");
-  if (istream == NULL)
+  if (! (STREQ (infile, "-") || freopen (infile, "r", stdin)))
     error (EXIT_FAILURE, errno, "%s", infile);
-
-  if (STREQ (outfile, "-"))
-    ostream = stdout;
-  else
-    ostream = fopen (outfile, "w");
-  if (ostream == NULL)
+  if (! (STREQ (outfile, "-") || freopen (outfile, "w", stdout)))
     error (EXIT_FAILURE, errno, "%s", outfile);
 
   thisline = &lb1;
@@ -297,11 +287,11 @@ check_file (const char *infile, const char *outfile)
       char *prevfield IF_LINT (= NULL);
       size_t prevlen IF_LINT (= 0);
 
-      while (!feof (istream))
+      while (!feof (stdin))
 	{
 	  char *thisfield;
 	  size_t thislen;
-	  if (readlinebuffer (thisline, istream) == 0)
+	  if (readlinebuffer (thisline, stdin) == 0)
 	    break;
 	  thisfield = find_field (thisline);
 	  thislen = thisline->length - 1 - (thisfield - thisline->buffer);
@@ -309,7 +299,7 @@ check_file (const char *infile, const char *outfile)
 	      || different (thisfield, prevfield, thislen, prevlen))
 	    {
 	      fwrite (thisline->buffer, sizeof (char),
-		      thisline->length, ostream);
+		      thisline->length, stdout);
 
 	      SWAP_LINES (prevline, thisline);
 	      prevfield = thisfield;
@@ -321,51 +311,56 @@ check_file (const char *infile, const char *outfile)
     {
       char *prevfield;
       size_t prevlen;
-      int match_count = 0;
-      int first_delimiter = 1;
+      uintmax_t match_count = 0;
+      bool first_delimiter = true;
 
-      if (readlinebuffer (prevline, istream) == 0)
+      if (readlinebuffer (prevline, stdin) == 0)
 	goto closefiles;
       prevfield = find_field (prevline);
       prevlen = prevline->length - 1 - (prevfield - prevline->buffer);
 
-      while (!feof (istream))
+      while (!feof (stdin))
 	{
 	  bool match;
 	  char *thisfield;
 	  size_t thislen;
-	  if (readlinebuffer (thisline, istream) == 0)
+	  if (readlinebuffer (thisline, stdin) == 0)
 	    {
-	      if (ferror (istream))
+	      if (ferror (stdin))
 		goto closefiles;
 	      break;
 	    }
 	  thisfield = find_field (thisline);
 	  thislen = thisline->length - 1 - (thisfield - thisline->buffer);
 	  match = !different (thisfield, prevfield, thislen, prevlen);
+	  match_count += match;
 
-	  if (match)
-	    ++match_count;
+	  if (match_count == UINTMAX_MAX)
+	    {
+	      if (count_occurrences)
+		error (EXIT_FAILURE, 0, _("too many repeated lines"));
+	      match_count--;
+	    }
 
           if (delimit_groups != DM_NONE)
 	    {
 	      if (!match)
 		{
 		  if (match_count) /* a previous match */
-		    first_delimiter = 0; /* Only used when DM_SEPARATE */
+		    first_delimiter = false; /* Only used when DM_SEPARATE */
 		}
 	      else if (match_count == 1)
 		{
 		  if ((delimit_groups == DM_PREPEND)
 		      || (delimit_groups == DM_SEPARATE
 			  && !first_delimiter))
-		    putc ('\n', ostream);
+		    putchar ('\n');
 		}
 	    }
 
 	  if (!match || output_later_repeated)
 	    {
-	      writeline (prevline, ostream, match, match_count);
+	      writeline (prevline, match, match_count);
 	      SWAP_LINES (prevline, thisline);
 	      prevfield = thisfield;
 	      prevlen = thislen;
@@ -374,30 +369,32 @@ check_file (const char *infile, const char *outfile)
 	    }
 	}
 
-      writeline (prevline, ostream, false, match_count);
+      writeline (prevline, false, match_count);
     }
 
  closefiles:
-  if (ferror (istream) || fclose (istream) == EOF)
-    error (EXIT_FAILURE, errno, _("error reading %s"), infile);
+  if (ferror (stdin) || fclose (stdin) != 0)
+    error (EXIT_FAILURE, 0, _("error reading %s"), infile);
 
-  if (ferror (ostream))
-    error (EXIT_FAILURE, 0, _("error writing %s"), outfile);
-  /* Close ostream only if it's not stdout -- the latter is closed
-     via the atexit-invoked close_stdout.  */
-  if (ostream != stdout && fclose (ostream) != 0)
-    error (EXIT_FAILURE, errno, _("error writing %s"), outfile);
+  /* stdout is handled via the atexit-invoked close_stdout function.  */
 
   free (lb1.buffer);
   free (lb2.buffer);
 }
 
+enum Skip_field_option_type
+  {
+    SFO_NONE,
+    SFO_OBSOLETE,
+    SFO_NEW
+  };
+
 int
 main (int argc, char **argv)
 {
   int optc = 0;
   bool posixly_correct = (getenv ("POSIXLY_CORRECT") != NULL);
-  bool obsolete_skip_fields = false;
+  enum Skip_field_option_type skip_field_option_type = SFO_NONE;
   int nfiles = 0;
   char const *file[2];
 
@@ -435,7 +432,7 @@ main (int argc, char **argv)
 	    break;
 	  if (nfiles == 2)
 	    {
-	      error (0, 0, _("extra operand `%s'"), argv[optind]);
+	      error (0, 0, _("extra operand %s"), quote (argv[optind]));
 	      usage (EXIT_FAILURE);
 	    }
 	  file[nfiles++] = argv[optind++];
@@ -452,7 +449,7 @@ main (int argc, char **argv)
 	      skip_chars = size;
 	    else if (nfiles == 2)
 	      {
-		error (0, 0, _("extra operand `%s'"), optarg);
+		error (0, 0, _("extra operand %s"), quote (optarg));
 		usage (EXIT_FAILURE);
 	      }
 	    else
@@ -471,12 +468,13 @@ main (int argc, char **argv)
 	case '8':
 	case '9':
 	  {
-	    size_t s = skip_fields;
-	    skip_fields = s * 10 + optc - '0';
-	    if (SIZE_MAX / 10 < s || skip_fields < s)
+	    if (skip_field_option_type == SFO_NEW)
+	      skip_fields = 0;
+
+	    if (!DECIMAL_DIGIT_ACCUMULATE (skip_fields, optc - '0', size_t))
 	      error (EXIT_FAILURE, 0, "%s",
 		     _("invalid number of fields to skip"));
-	    obsolete_skip_fields = true;
+	    skip_field_option_type = SFO_OBSOLETE;
 	  }
 	  break;
 
@@ -499,16 +497,17 @@ main (int argc, char **argv)
 					delimit_method_map);
 	  break;
 
-	case 'f':		/* Like '-#'. */
+	case 'f':
+	  skip_field_option_type = SFO_NEW;
 	  skip_fields = size_opt (optarg,
 				  N_("invalid number of fields to skip"));
 	  break;
 
 	case 'i':
-	  ignore_case = 1;
+	  ignore_case = true;
 	  break;
 
-	case 's':		/* Like '+#'. */
+	case 's':
 	  skip_chars = size_opt (optarg,
 				 N_("invalid number of bytes to skip"));
 	  break;
@@ -531,13 +530,6 @@ main (int argc, char **argv)
 	}
     }
 
-  if (obsolete_skip_fields && 200112 <= posix2_version ())
-    {
-      error (0, 0, _("`-%lu' option is obsolete; use `-f %lu'"),
-	     (unsigned long) skip_fields, (unsigned long) skip_fields);
-      usage (EXIT_FAILURE);
-    }
-
   if (countmode == count_occurrences && output_later_repeated)
     {
       error (0, 0,
